package es.uca.xmleye.presentacion;

/*
 Copyright (C) 2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.RandomAccessFile;
import java.nio.channels.FileLock;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentLinkedQueue;
import java.util.concurrent.ExecutionException;
import java.util.Queue;
import java.util.Locale;
import java.util.ResourceBundle;

import static es.uca.xmleye.ConstantesAplicacion.*;
import es.uca.xmleye.Aplicacion;
import es.uca.xmleye.ExcepcionAnalisis;
import es.uca.xmleye.ExcepcionImportacion;
import es.uca.xmleye.ModeloDocumento;
import es.uca.xmleye.prefs.*;
import es.uca.xmleye.tipos.*;
import es.uca.xmleye.util.ConstructorOrdenes;
import es.uca.xmleye.util.Contenedor;
import es.uca.xmleye.util.ProcesoFiltro;
import es.uca.xmleye.util.TareaEntrada;
import es.uca.xmleye.util.TareaVolcado;
import es.uca.xmleye.xslt.*;

/**
 * Modelo de presentación de la aplicación completa.
 */
public class ModPresentacionVentanaPrincipal {
	private static final ResourceBundle errRes;
	private static final ResourceBundle miscRes;

	static {
		miscRes = ResourceBundle.getBundle("es.uca.xmleye.l10n.MiscMessages",
				Locale.getDefault());
		errRes = ResourceBundle.getBundle("es.uca.xmleye.l10n.ErrorMessages",
				Locale.getDefault());
	}

	private GestorPreferencias gestorPrefs; // Preferencias actuales
	private RepositorioHojas repoHojas; // Hojas disponibles

	// Documentos abiertos actualmente, según fichero origen
	private Map<File, ModPresentacionDocumento> docsAbiertos;

	// Tipos disponibles
	private ModPresentacionVentanaTipos modTipos;

	private Historial docsRecientes; // Historial de documentos recientes
	private PrefBoolean bActAutomatica; // ¿Actualización automática?

	private HojaPreprocesado hojaPreprocOmision; // Hoja de preprocesado por
	// omisión
	private HojaVisualizacion hojaVistaOmision; // Hoja de visualización por
	// omisión

	private Queue<Contenedor<String>> mensajesTareas; // Mensajes de estado de

	// las tareas en
	// ejecución y
	// pendientes

	/* --- CONSTRUCTORES --- */
	public ModPresentacionVentanaPrincipal(ModPresentacionVentanaTipos modTipos)
			throws ExcepcionAnalisis, ExcepcionTransformacion {

		this.modTipos = modTipos;
		this.gestorPrefs = GestorPreferenciasSingleton.getInstancia();
		this.docsAbiertos = new HashMap<File, ModPresentacionDocumento>();
		this.repoHojas = new RepositorioHojas(ConstantesHojas.DIR_HOJAS);

		this.docsRecientes = new Historial();
		this.bActAutomatica = new PrefBoolean(gestorPrefs,
				ConstantesPreferencias.CLAVE_AUTOIMPORT,
				ConstantesPreferencias.INICIAL_AUTOIMPORT);

		this.hojaPreprocOmision = repoHojas.getHojaPreprocesado(gestorPrefs
				.getPrefEscalar(
						ConstantesPreferencias.CLAVE_PREPROCESADO_OMISION,
						ConstantesPreferencias.INICIAL_PREPROCESADO_OMISION));
		this.hojaVistaOmision = repoHojas.getHojaVisualizacion(gestorPrefs
				.getPrefEscalar(ConstantesPreferencias.CLAVE_VISTA_OMISION,
						ConstantesPreferencias.INICIAL_VISTA_OMISION));

		this.mensajesTareas = new ConcurrentLinkedQueue<Contenedor<String>>();
	}

	/* ------------------------- METODOS PUBLICOS ---------------------------- */

	/* -------- Accesores para sincronización -------- */
	public List<String> getEntradasHistorial() {
		return docsRecientes.getHistorial();
	}

	public List<String> getNombresHojasPreproc() {
		return repoHojas.getNombresHojasPreprocesado();
	}

	public List<String> getNombresHojasVista() {
		return repoHojas.getNombresHojasVisualizacion();
	}

	public boolean getActualizacionAutomatica() {
		return bActAutomatica.get();
	}

	public String getTitulo(ModPresentacionDocumento docActual) {
		return "XMLEye "
				+ Aplicacion.VERSION
				+ ((docActual.getNombreDocumento() != null) ? (" - " + docActual
						.getNombreDocumento())
						: "");
	}

	public boolean getOcupado() {
		return !mensajesTareas.isEmpty();
	}

	public String getMensajeEstado() {
		synchronized (mensajesTareas) {
			if (mensajesTareas.isEmpty()) {
				return miscRes.getString("status_ready");
			} else {
				return mensajesTareas.peek().get();
			}
		}
	}

	/* --- Respuesta a eventos --- */

	// / Lanzamiento de solicitud de apertura de un fichero
	public ModPresentacionDocumento solicitudAbrirDocumento(File fich)
			throws ExcepcionAnalisis, ExcepcionTransformacion,
			ExcepcionImportacion {
		final HojaPreprocesado hojaPreproc;
		final HojaVisualizacion hojaVista;
		final Contenedor<String> elem;

		synchronized (this) {
			hojaPreproc = hojaPreprocOmision;
			hojaVista = hojaVistaOmision;
			elem = inicioTarea(miscRes.getString("status_opening"));
		}

		// Lo quitamos del historial por si hay un fallo
		docsRecientes.retirar(fich.getAbsolutePath());

		try {
			final Tipo tipo = modTipos.getTipoPorFich(fich);

			if (tipo != null && tipo.getOrdenImportacion() != null
					&& tipo.getOrdenImportacion().trim().length() > 0) {

				// Invocaremos al importador y sustituiremos fich
				File fichTmp = File.createTempFile("gen", null);
				finTarea(elem);

				ProcesoFiltro proc = new ProcesoFiltro(ConstructorOrdenes
						.prepararOrden(tipo.getOrdenImportacion(),
								new String[] { CLAVE_FUENTE },
								new String[] { fich.getAbsolutePath() }), null,
						new TareaVolcado(fichTmp), new TareaLineasAEstado());

				int codigoProc = proc.iniciar(true);
				if (codigoProc != 0) {
					throw new ExcepcionImportacion(String.format(errRes
							.getString("err_importer_fail"), codigoProc));
				}

				ModPresentacionDocumento res = new ModPresentacionDocumento(
						new ModeloDocumento(fichTmp, hojaPreproc, fich
								.getAbsoluteFile().getParent()), hojaVista,
						this);

				return res;
			} else {

				ModPresentacionDocumento res = new ModPresentacionDocumento(
						new ModeloDocumento(fich, hojaPreproc, fich
								.getAbsoluteFile().getParent()), hojaVista,
						this);

				return res;
			}

		} catch (FileNotFoundException ex) {
			throw new ExcepcionImportacion(ex);
		} catch (IOException ex) {
			throw new ExcepcionImportacion(String.format(errRes
					.getString("err_import_ioex"), ex.getLocalizedMessage()),
					ex);
		} catch (InterruptedException ex) {
			throw new ExcepcionImportacion(ex);
		} catch (ExcepcionAnalisis ex) {
			throw ex;
		} catch (ExcepcionTransformacion ex) {
			throw ex;
		} catch (ExecutionException e) {
			throw new ExcepcionImportacion(e);
		} finally {
			if (elem != null) {
				finTarea(elem);
			}
		}
	}

	// / Lanzamiento de solicitud de reapertura de un fichero
	public ModPresentacionDocumento solicitudReabrirDocumento(
			ModPresentacionDocumento antiguo) throws ExcepcionAnalisis,
			ExcepcionTransformacion, ExcepcionImportacion {

		ModPresentacionDocumento nuevoDoc = solicitudAbrirDocumento(antiguo
				.getFicheroFuente());
		return nuevoDoc;
	}

	// / Añadido de un documento
	public synchronized boolean agregarDocumento(
			ModPresentacionDocumento nuevoDoc) throws ExcepcionTransformacion {

		nuevoDoc.cambioAutoImportacion(bActAutomatica.get());
		File ficheroOrigen = nuevoDoc.getFicheroOrigen();
		docsRecientes.agregar(ficheroOrigen.getAbsolutePath());

		if (docsAbiertos.containsKey(ficheroOrigen)) {
			docsAbiertos.get(ficheroOrigen).setModeloDocumento(
					nuevoDoc.getModeloDocumento());
			nuevoDoc.cerrar();
			return false;
		} else {
			docsAbiertos.put(ficheroOrigen, nuevoDoc);
			return true;
		}
	}

	// / Cierre de un documento por su nombre
	public synchronized void cierreDocumento(File fCerrado) {
		for (File f : docsAbiertos.keySet()) {
			if (f.equals(fCerrado)) {
				cierreDocumento(docsAbiertos.get(f));
				return;
			}
		}
	}

	// / Cierre del documento actual
	public synchronized void cierreDocumento(ModPresentacionDocumento docCerrado) {
		// Interrumpimos el vigilado del fichero
		docCerrado.getNotCambios().interrumpir();
		// Retiramos de la lista de abiertos
		docsAbiertos.remove(docCerrado.getFicheroOrigen());
	}

	// / Obtención de la lista de ficheros abiertos
	public List<File> getDocumentosAbiertos() {
		ArrayList<File> vFicheros = new ArrayList<File>();
		for (File f : docsAbiertos.keySet()) {
			vFicheros.add(f);
		}
		return vFicheros;
	}

	// / Obtiene la vista para un título determinado
	public ModPresentacionDocumento getModeloDocumento(File fich) {
		if (docsAbiertos.containsKey(fich)) {
			return docsAbiertos.get(fich);
		}
		return null;
	}

	// / Cambio de la hoja de preprocesado por defecto para nuevos documentos
	public synchronized void cambioHojaPreprocesado(
			ModPresentacionDocumento docActual, String nombreNuevaHoja)
			throws ExcepcionTransformacion, ExcepcionAnalisis {
		// Si es la misma de antes, no hacemos nada
		if (nombreNuevaHoja.equals(hojaPreprocOmision.getNombre()))
			return;

		// Si la hoja existe, cambiamos la hoja por defecto
		HojaPreprocesado nuevaHoja = repoHojas
				.getHojaPreprocesado(nombreNuevaHoja);
		if (nuevaHoja != null) {
			hojaPreprocOmision = nuevaHoja;
			gestorPrefs.setPrefEscalar(
					ConstantesPreferencias.CLAVE_PREPROCESADO_OMISION,
					nombreNuevaHoja);

			docActual.cambioHojaPreprocesado(nuevaHoja);
		}
	}

	// / Cambio de la hoja de visualización por defecto para nuevos documentos y
	// para el actual
	public synchronized void cambioHojaVisualizacion(
			ModPresentacionDocumento docActual, String nombreNuevaHoja)
			throws ExcepcionTransformacion {
		// Si es la misma de antes, no hacemos nada
		if (nombreNuevaHoja.equals(hojaVistaOmision.getNombre()))
			return;

		// Si la hoja existe, cambiamos la hoja por defecto
		HojaVisualizacion nuevaHoja = repoHojas
				.getHojaVisualizacion(nombreNuevaHoja);
		if (nuevaHoja != null) {
			hojaVistaOmision = nuevaHoja;
			gestorPrefs
					.setPrefEscalar(ConstantesPreferencias.CLAVE_VISTA_OMISION,
							nombreNuevaHoja);

			docActual.cambioHojaVisualizacion(nuevaHoja);
		}
	}

	// / Cambio de la autoimportación
	public synchronized void cambioAutoImportacion() {
		// Cambiamos de valor la actualizacion automatica
		final boolean nuevoValor = !bActAutomatica.get();
		fireSetAutoImportacion(nuevoValor);
	}

	private void fireSetAutoImportacion(final boolean nuevoValor) {
		bActAutomatica.set(nuevoValor);

		// Avisamos a los modelos de documento
		for (ModPresentacionDocumento doc : docsAbiertos.values()) {
			doc.cambioAutoImportacion(nuevoValor);
		}
	}

	/* --- METODOS DE PAQUETE --- */
	Contenedor<String> inicioTarea(String msg) {
		Contenedor<String> elem = new Contenedor<String>(msg);
		mensajesTareas.offer(elem);
		return elem;
	}

	String getOrdenEditor(File f) {
		return modTipos.getTipoPorFich(f).getOrdenEdicion();
	}

	void finTarea(Contenedor<String> elem) {
		mensajesTareas.remove(elem);
	}

	/** CLASES INTERNAS * */

	private class TareaLineasAEstado implements TareaEntrada {
		private InputStream is;

		public void setFlujoEntrada(InputStream is) {
			this.is = is;
		}

		public Integer call() throws Exception {
			final BufferedReader bIn = new BufferedReader(
					new InputStreamReader(is));

			int nlineas = 0;
			String linea = bIn.readLine();

			if (linea != null) {
				Contenedor<String> lineaAnterior = inicioTarea(linea);

				while ((linea = bIn.readLine()) != null) {
					synchronized (mensajesTareas) {
						finTarea(lineaAnterior);
						lineaAnterior = inicioTarea(linea);
					}

					++nlineas;
				}

				finTarea(lineaAnterior);
			}

			bIn.close();
			return nlineas;
		}
	}

}
