package es.uca.xmleye.util;

/*
 Copyright (C) 2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.io.File;
import java.util.HashSet;
import java.util.Set;

import es.uca.xmleye.eventos.EventoCambioFichero;
import es.uca.xmleye.eventos.ObservadorCambioFichero;

public class NotificadorCambiosFichero {
	// Por defecto, cada 0.1 segundos
	public static final int INTERVALO_OMISION = 100;
	
	public static final boolean ACTIVO   = true;
	public static final boolean INACTIVO = false;

	private File fichVigilado;
	private long ultModificacion;
	private int intervaloEspera;
	private Set<ObservadorCambioFichero> observadores;
	private Thread hiloVigilado;
	private boolean activo;

	/**
	 * Crea un nuevo notificador.
	 * 
	 * @param f
	 *            Fichero a vigilar.
	 */
	public NotificadorCambiosFichero(File f) {
		this(f, INTERVALO_OMISION, true);
	}

	/**
	 * Crea un nuevo notificador.
	 * 
	 * @param f
	 *            Fichero a vigilar.
	 * @param intervalo
	 *            Intervalo de comprobación en milisegundos.
	 */
	public NotificadorCambiosFichero(File f, int intervalo) {
		this(f, intervalo, true);
	}
	
	/**
	 * Crea un nuevo notificador.
	 * 
	 * @param f
	 *            Fichero a vigilar.
	 * @param intervalo
	 *            Intervalo de comprobación en milisegundos.
	 * @param activo
	 *            Especifica si debería estar inicialmente activo.
	 */
	public NotificadorCambiosFichero(File f, int intervalo, boolean activo) {
		this.fichVigilado = f;
		this.ultModificacion = f.lastModified();
		this.intervaloEspera = intervalo;
		this.observadores = new HashSet<ObservadorCambioFichero>();
		this.activo = activo;
		
		hiloVigilado = new Thread(new Runnable() {
			public void run() {
				vigilarFichero();
			}
		});

		hiloVigilado.setPriority(Thread.MIN_PRIORITY);
		hiloVigilado.start();
	}

	/**
	 * Registra a un nuevo observador. Se evitan duplicados.
	 */
	public void agregarObservadorCambioFichero(ObservadorCambioFichero ob) {
		observadores.add(ob);
	}

	/**
	 * Retira a un observador. No pasa nada si no estaba ya.
	 */
	public void retirarObservadorCambioFichero(ObservadorCambioFichero ob) {
		observadores.remove(ob);
	}

	/**
	 * Interrumpe el vigilado del fichero.
	 */
	public void interrumpir() {
		hiloVigilado.interrupt();
	}

	/**
	 * Pausa el vigilado del fichero.
	 */
	public void pausar() {
		activo = false;
	}

	/**
	 * Reanuda el vigilado del fichero.
	 */
	public synchronized void reanudar() {
		activo = true;
		notifyAll();
	}

	/** --- METODOS PROTEGIDOS --- */

	protected void fireFicheroCambiado(EventoCambioFichero e) {
		// Hacemos una copia de los elementos de la cola
		HashSet<ObservadorCambioFichero> obs;
		synchronized (observadores) {
			obs = new HashSet<ObservadorCambioFichero>(observadores);
		}

		for (ObservadorCambioFichero ob : obs) {
			ob.ficheroCambiado(e);
		}
	}

	/** --- METODOS PRIVADOS --- */

	/**
	 * Vigila el fichero en busca de cambios.
	 */
	private void vigilarFichero() {
		try {
			while (true) {
				synchronized (this) {
					// Permite combinar la pausa y la comprobación de la fecha
					while (!activo
							|| ultModificacion == fichVigilado.lastModified()
							|| !fichVigilado.canRead()) {
						wait(intervaloEspera);
					}

					this.ultModificacion = fichVigilado.lastModified();
					fireFicheroCambiado(new EventoCambioFichero(fichVigilado,
							this, fichVigilado.lastModified()));
				}
			}
		} catch (InterruptedException ex) {
		}
	}

}
