package es.uca.xmleye.xslt;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.io.*;
import javax.xml.transform.stream.*;
import javax.xml.transform.*;

/**
 * <p>
 * URIResolver muy sencillo: sólo intenta aplicar un determinado directorio base
 * a todas las URI que encuentra. Si el fichero existe, devuelve dicha URI. Usa
 * una ruta absoluta o una ruta relativa respecto del directorio actual.
 * </p>
 * 
 * <p>
 * Decora un resolvedor subordinado al que recurre cuando no es capaz de
 * resolver una URI.
 * </p>
 * 
 * <p>
 * <b>NOTA</b>: por lo pronto, no se usan las URI base en la resolución.
 * </p>
 * 
 * @author Antonio García Domínguez
 * @version 1.0
 */
class ResolvedorDirBase implements URIResolver {
	final private String dirBase;
	private URIResolver subordinado;

	/**
	 * Crea un nuevo resolvedor.
	 * 
	 * @param dirBase
	 *            Directorio base que aplicar a todas las URI.
	 * @param subordinado
	 *            Resolvedor subordinado al que recurrir si una URI no se puede
	 *            resolver.
	 * @exception Lanza
	 *                NullPointerException si <code>dirBase</code> o
	 *                <code>subordinado</code> son <code>null</code>.
	 */
	public ResolvedorDirBase(String dirBase, URIResolver subordinado) {
		assert(dirBase != null);
		assert(subordinado != null);
		
		this.dirBase = dirBase;
		this.subordinado = subordinado;
	}

	/**
	 * Devuelve el directorio base.
	 * 
	 * @return Directorio base aplicado a todas las URI.
	 */
	public String getDirBase() {
		return dirBase;
	}

	public Source resolve(String href, String base) {
		try {
			File f = new File(dirBase + File.separator + href);
			return new StreamSource(new BufferedInputStream(
					new FileInputStream(f)));
		} catch (FileNotFoundException ex) {
			try {
				return subordinado.resolve(href, base);
			} catch (TransformerException t_ex) {
				System.err.println(t_ex.getMessage());
				return null;
			}
		}
	}
}